#  zzz.R
#  FBA and friends with R.
#
#  Copyright (C) 2010-2012 Gabriel Gelius-Dietrich, Dpt. for Bioinformatics,
#  Institute for Informatics, Heinrich-Heine-University, Duesseldorf, Germany.
#  All right reserved.
#  Email: geliudie@uni-duesseldorf.de
#
#  This file is part of SyBiL.
#
#  SyBiL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SyBiL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SyBiL.  If not, see <http://www.gnu.org/licenses/>.


.SYBILenv <- new.env()

.onLoad <- function(lib, pkg) {

    # -------------------------------------------------------------- #
    # stacks and queues in SyBiL

    .SYBILenv$sybilStack <- vector(mode = "list", length = 0)
    

    # -------------------------------------------------------------- #
    # settings in SyBiL

    .SYBILenv$settings <- list(
        SOLVER            = "glpk",
        METHOD            = "simplex",
        TOLERANCE         = 1E-6,
        MAXIMUM           = 1000,
        ALGORITHM         = "FBA",
        OPT_DIRECTION     = "max",
        PATH_TO_MODEL     = ".",
        SOLVER_CTRL_PARAM = as.data.frame(NA)
    )


    # -------------------------------------------------------------- #
    # available solvers and methods

    # The first method in each vector is the default one, the default
    # alternate method is NA.

    .SYBILenv$solverMethods <- list(
        glpk       = c("simplex", "interior", "exact"),

        clp        = c("general_solve", "inidual", "iniprimal", "inibarrier",
                       "inibarriernoc", "idiot", "dual", "primal"),

        lpSolveAPI = c("lp_solve"),

        cplex      = c("lpopt", "primopt", "dualopt", "baropt", "hybbaropt",
                       "hybnetopt", "siftopt")
    )

    .SYBILenv$solvers <- c("glpk", "clp", "lpSolveAPI", "cplex")

    .SYBILenv$solverCtrlParam <- list(
        glpk = list(
                    simplex  = as.data.frame(NA),
                    interior = as.data.frame(NA),
                    exact    = as.data.frame(NA)
        ),

        clp = list(
                   general_solve = as.data.frame(NA),
                   inidual       = as.data.frame(NA),
                   iniprimal     = as.data.frame(NA),
                   inibarrier    = as.data.frame(NA),
                   inibarriernoc = as.data.frame(NA),
                   dual          = as.data.frame(NA),
                   primal        = as.data.frame(NA)
        ),

        lpSolveAPI = list(
                          lp_solve = as.data.frame(NA)
        ),

        cplex = list(
                     lpopt     = as.data.frame(NA),
                     primopt   = as.data.frame(NA),
                     #primopt   = data.frame(
                     #   #CPX_PARAM_REDUCE    = as.integer(1),
                     #    CPX_PARAM_DATACHECK = as.integer(1)
                     #),
                     dualopt   = as.data.frame(NA),
                     baropt    = as.data.frame(NA),
                     hybbaropt = as.data.frame(NA),
                     hybnetopt = as.data.frame(NA),
                     siftopt   = as.data.frame(NA)
        )
    )
}


